@@ -45,7 +45,7 @@ SEXP unwind_protect(Fun&& code) {
     throw unwind_exception(token);
   }
 
-  return R_UnwindProtect(
+  SEXP res = R_UnwindProtect(
       [](void* data) -> SEXP {
         auto callback = static_cast<decltype(&code)>(data);
         return static_cast<Fun&&>(*callback)();
@@ -59,6 +59,14 @@ SEXP unwind_protect(Fun&& code) {
         }
       },
       &jmpbuf, token);
+
+  // R_UnwindProtect adds the result to the CAR of the continuation token,
+  // which implicitly protects the result. However if there is no error and
+  // R_UwindProtect does a normal exit the memory shouldn't be protected, so we
+  // unset it here before returning the value ourselves.
+  SETCAR(token, R_NilValue);
+
+  return res;
 }
 
 template <typename Fun, typename = typename std::enable_if<std::is_same<
@@ -188,22 +196,22 @@ inline void check_user_interrupt() { safe[R_CheckUserInterrupt](); }
 
 template <typename... Args>
 void stop [[noreturn]] (const char* fmt, Args... args) {
-  safe.noreturn(Rf_error)(fmt, args...);
+  safe.noreturn(Rf_errorcall)(R_NilValue, fmt, args...);
 }
 
 template <typename... Args>
 void stop [[noreturn]] (const std::string& fmt, Args... args) {
-  safe.noreturn(Rf_error)(fmt.c_str(), args...);
+  safe.noreturn(Rf_errorcall)(R_NilValue, fmt.c_str(), args...);
 }
 
 template <typename... Args>
 void warning(const char* fmt, Args... args) {
-  safe[Rf_warning](fmt, args...);
+  safe[Rf_warningcall](R_NilValue, fmt, args...);
 }
 
 template <typename... Args>
 void warning(const std::string& fmt, Args... args) {
-  safe[Rf_warning](fmt.c_str(), args...);
+  safe[Rf_warningcall](R_NilValue, fmt.c_str(), args...);
 }
 
 /// A doubly-linked list of preserved objects, allowing O(1) insertion/release of
